//+------------------------------------------------------------------+
//|                                                    GnomeSort.mqh |
//|                                    2019-2021, dimitri pecheritsa |
//|                                         mql5.com/en/users/dmipec |
//|------------------------------------------------------------------|
//|  c  | gnome sort                                                 |
//|------------------------------------------------------------------|
//| use | array sorting algorithm                                    |
//|  best: n; average: n^2; worst: n^2                               |
//|  memory: 1; stable: yes; method: exchanging                      |
//|  note: tiny code size                                            |
//|  originally proposed by iranian computer scientist hamid         |
//|sarbazi-azad (professor of computer science and engineering at    |
//|sharif university of technology) in 2000. the sort was first      |
//|called stupid sort (not to be confused with bogosort), and then   |
//|later described by dick grune and named gnome sort.               |
//|  the gnome sort is a sorting algorithm which is similar to       |
//|insertion sort in that it works with one item at a time but gets  |
//|the item to the proper place by a series of swaps, similar to a   |
//|bubble sort. it is conceptually simple, requiring no nested loops.|
//|the average running time is o(n^2) but tends towards o(n) if the  |
//|list is initially almost sorted.                                  |
//|  the algorithm finds the first place where two adjacent elements |
//|are in the wrong order and swaps them. it takes advantage of the  |
//|fact that performing a swap can introduce a new out-of-order      |
//|adjacent pair next to the previously swapped elements. it does not|
//|assume that elements forward of the current position are sorted,  |
//|so it only needs to check the position directly previous to the   |
//|swapped elements.                                                 |
//+------------------------------------------------------------------+
#include <Mqh\Algorithms\ASorter.mqh>
template<typename TKey,typename TItem>
class CGnomeSort : public ASorter<TKey,TItem>
  {
public:
                     CGnomeSort(void) {}
                    ~CGnomeSort(void) {}
   void              Sort(void);
  };
//+------------------------------------------------------------------+
//|  m  | sort                                                       |
//|------------------------------------------------------------------|
//| use | gnome sort is based on the technique used by the standard  |
//|dutch garden gnome.                                               |
//|  here is how a garden gnome sorts a line of flower pots.         |
//|  basically, he looks at the flower pot next to him and the       |
//|previous one; if they are in the right order he steps one pot     |
//|forward, otherwise, he swaps them and steps one pot backward.     |
//|  boundary conditions: if there is no previous pot, he steps      |
//|forwards; if there is no pot next to him, he is done.             |
//+------------------------------------------------------------------+
#include <Mqh\Universal\CompareFunction.mqh>
template<typename TKey,typename TItem>
void CGnomeSort::Sort(void)
  {
   for(int pos=0; pos<ArraySize(keys);)
     {
      if(pos==0 || Compare(keys[pos],keys[pos-1])!=-1)
         pos++;
      else
        {
         Swap(pos,pos-1);
         pos--;
        }
     }
  }
//+------------------------------------------------------------------+
